(*
 *
 *)

unit
   MissingInFPC;

// ============================================================================================
// ============================================================================================

interface

uses
   Windows;

// --------------------------------------------------------------------------------------------
// --- from: SysUtils.pas                                                      ----------------

var
{ Win32 platform identifier.  This will be one of the following values:
    VER_PLATFORM_WIN32s
    VER_PLATFORM_WIN32_WINDOWS
    VER_PLATFORM_WIN32_NT
  See WINDOWS.PAS for the numerical values. }

   Win32Platform: Integer = 0;

{ Win32 OS version information -
  See TOSVersionInfo.dwMajorVersion/dwMinorVersion/dwBuildNumber }

   Win32MajorVersion: Integer = 0;
   Win32MinorVersion: Integer = 0;
   Win32BuildNumber: Integer = 0;

{ Win32 OS extra version info string -
  See TOSVersionInfo.szCSDVersion }

   Win32CSDVersion: String = '';

{ SafeLoadLibrary calls LoadLibrary, disabling normal Win32 error message
  popup dialogs if the requested file can't be loaded.  SafeLoadLibrary also
  preserves the current FPU control word (precision, exception masks) across
  the LoadLibrary call (in case the DLL you're loading hammers the FPU control
  word in its initialization, as many MS DLLs do). }

function SafeLoadLibrary (const Filename: String; ErrorMode: UINT): HMODULE;

// --------------------------------------------------------------------------------------------
// --- from: Windows.pas                                                       ----------------

function UnlockResource (hResData: THandle): BOOL;

// ============================================================================================
// ============================================================================================

implementation

// --------------------------------------------------------------------------------------------
// --- from: SysUtils.pas                                                      ----------------

// --------------------------------------------------------------------------------------------
function SafeLoadLibrary (const Filename: String; ErrorMode: UINT): HMODULE;
var
   OldMode: UINT;
   FPUControlWord: Word;
begin
   OldMode := SetErrorMode(ErrorMode);
   try
      asm
         FNSTCW FPUControlWord
      end;
      try
         Result := LoadLibrary(PChar(Filename));
      finally
         asm
            FNCLEX
            FLDCW FPUControlWord
         end;
      end;
   finally
      SetErrorMode(OldMode);
   end;
end;

// --------------------------------------------------------------------------------------------
// --- from: Windows.pas                                                       ----------------

// --------------------------------------------------------------------------------------------
function UnlockResource (hResData: THandle): BOOL;
begin
   if (hResData > 0) then
      ;
   Result := False;
end;

// ============================================================================================
// ============================================================================================

// --------------------------------------------------------------------------------------------
// --- from: SysUtils.pas                                                      ----------------

// --------------------------------------------------------------------------------------------
procedure InitPlatformId ();
var
   OSVersionInfo: TOSVersionInfo;
begin
   OSVersionInfo.dwOSVersionInfoSize := SizeOf(OSVersionInfo);
   if GetVersionEx(OSVersionInfo) then
      with OSVersionInfo do begin
         Win32Platform := dwPlatformId;
         Win32MajorVersion := dwMajorVersion;
         Win32MinorVersion := dwMinorVersion;
         Win32BuildNumber := dwBuildNumber;
         Win32CSDVersion := szCSDVersion;
      end;
end;

// ============================================================================================
// ============================================================================================

initialization

   // -----------------------------------------------------------------------------------------
   // --- from: SysUtils.pas                                                   ----------------

   InitPlatformId();

end.

