(*
 *                                                                                             .
 * Module name: sv.dpr; Module type: Delphi (console) project; Language: Object pascal;        .
 * Original developed with: Delphi 5 Enterprise; Portability: most likely down to Delphi 2;    .
 *                                                                                             .
 * Author: ~SoffyT (web:<http://www.nefzger.at/~SoffyT/>; mailto:<MissyQueen@nefzger.at>)      .
 * File created: Fri., 2003-08-15; Last modified: Sat., 2005-10-22                             .
 *                                                                                             .
 * Bug report: Pls. report all bugs, wishes, suggestions etc. to <BugReport@nefzger.at>.       .
 *                                                                                             .
 *  Purpose:                                                                                   .
 * ----------                                                                                  .
 *    Console wrapper for 'Sieve.pas'                                                          .
 *                                                                                             .
 *)

(*
 *                                                                                             .
 * To compile with Delphi:                                                                     .
 *    (To do ..)                                                                               .
 * To compile with FPC:                                                                        .
 *    [ windres -i sv.verinfo.res -o sv.verinfo.or -J res -O res -F pe-i386 ]                  .
 *    [ i:\brc32 -v -32 -r -fosv.verinfo.res sv.verinfo.rc ]                                   .
 *    fpc -B -Sd -Op2 -Xs -XX -Fu"..\includes;..\core_utilities" -FU"%TEMP%" sv.dpr -osv.fpc.exe   .
 *                                                                                             .
 *)

{$APPTYPE Console}
program
   SieveOfEratosthenes; // 'sv' or 'sv.dpr'                                                    .

{$INCLUDE '..\includes\Globals.inc'}

uses
   Windows,
   SysUtils,
   Math,
{$IFDEF DelphiPascalCompiler}
   CmdLine_Utilities in '..\core_utilities\CmdLine_Utilities.pas',
   Misc_Utilities in '..\core_utilities\Misc_Utilities.pas';
{$ENDIF}
{$IFDEF FreePascalCompiler}
   CmdLine_Utilities,
   Misc_Utilities;
{$ENDIF}

// ----------------------------------------------------------------------------                .
function Eval_CmdLine (var A_cnt: Integer; var be_quiet: Boolean): Boolean;
begin
   if (ParamCount() >= 1) then begin
      A_cnt := StrToIntDef(ParamStr(1),(-1));
   end else begin
      A_cnt := (-1);
   end;
   be_quiet := (ParamCount() = 2) and
               ((CmdLineSwitch('q',['-','/'],False{$IFDEF FreePascalCompiler},False{$ENDIF}) > 0) or
                (CmdLineSwitch('quiet',['-'],False,True) > 0));
   Result := (A_cnt > (-1)) and
             ((ParamCount() = 1) or (be_quiet));
end;

// ----------------------------------------------------------------------------                .
function Num_Digits (X: Extended): Integer;
begin
   X := Log10(X);
   if (X = Ceil(X)) then begin
      Result := (Ceil(X) + 1);
   end else begin
      Result := Ceil(X);
   end;
end;

// -- Programm- & Versions-Info -------------------------------                                .
{$IFDEF DelphiPascalCompiler}
{$R 'sv.verinfo.res'}          // generiert mit VerGen.exe Copyright (c) 2002, 2005 ~MNe :-)   .
{$ENDIF}
{$IFDEF FreePascalCompiler}
{$R 'sv.verinfo.or'}           // generiert mit VerGen.exe Copyright (c) 2002, 2005 ~MNe :-)   .
{$ENDIF}
var
   Program_name : String;
   CmdLine_ok : Integer;
// ------------------------------------------------------------                                .

const
   MIN_N                  = 2;
   MAX_N                  = (256 * 1024 * 1024); // Max. Primzahl: 2^28 ..                     .
// => GetConsoleScreenBufferInfo()                                                             .
   SCREEN_WIDTH : Integer = 80;  // Anzahl der Zeichen pro Bildschirm-Zeile ..                 .

var
   A : TSieveResultArray;
   A_to, A_cnt, p_cnt, max_width, i : Integer;
   be_quiet : Boolean;

// ============================================================================                .

begin try
   // Vorspann des Programms                                                                   .
   Program_name := appGetApplicationFileName();
   CmdLine_ok := GetFirstInvalidSwitch(
      CheckCmdLineSwitches(  // Kommandozeile auswerten #1, "Lang-Form" ..                     .
         ['version','history',
          'quiet'],
         ['-'],
         [False,False,
          False],
         True),
      CheckCmdLineSwitches(  // Kommandozeile auswerten #2, "Kurz-Form" ..                     .
         ['v','h',
          'q'],
         ['-','/'],
         [False,False,
          False]
         {$IFDEF FreePascalCompiler},False{$ENDIF}));
   WriteLn;
   WriteLn(Program_name,' -- ',resGetXVersionInfo(),' (',resGetXWersionInfo(),')',', ',
      resGetXReleaseInfo(),', ',resGetXDateTimeInfo());
   WriteLn(StringOfChar(' ',Length(Program_name)),' -- ',resGetXCopyrightInfo());
   WriteLn(StringOfChar(' ',Length(Program_name)),' -- ','(dbg) OS: ',
      sysGetWin32Version().w32ComposedName,'; CmdLineCnt: ',ParamCount(),'; CmdLineErg: ',
      CmdLine_ok);
   WriteLn;
   // Kommandozeile auswerten                                                                  .
   if (CmdLine_ok = CMDLINE_VALID) and
      ((ParamCount = 1) and ((CmdLineSwitch('v',['-','/'],False{$IFDEF FreePascalCompiler},False{$ENDIF}) > 0) or
                             (CmdLineSwitch('version',['-'],False,True) > 0))) then begin
      // Parameter '--version': keine weiteren Informationen anzeigen ..                       .
      {$INCLUDE '.\all.motd.inc'}
   end else if (CmdLine_ok = CMDLINE_VALID) and
               ((ParamCount = 1) and ((CmdLineSwitch('h',['-','/'],False{$IFDEF FreePascalCompiler},False{$ENDIF}) > 0) or
                                      (CmdLineSwitch('history',['-'],False,True) > 0))) then begin
      // Parameter '-History': als Resource gelinkte History anzeigen ..                       .
      WriteLn(resGetXHistoryInfo());
   end else if (CmdLine_ok = CMDLINE_VALID) and
               ((Eval_CmdLine(A_to,be_quiet)) and
                ((A_to >= MIN_N) and (A_to <= MAX_N))) then begin
   // --                                                                                       .
      if not(be_quiet) then begin
         Write('Calculating sv(',A_to,'). Please wait ..');
      end;
   // --                                                                                       .
      A_cnt := Sieve_Of_Eratosthenes(A_to,A);
   // --                                                                                       .
      if not(be_quiet) then begin
         WriteLn(' done.');
         WriteLn;
      end;
   // --                                                                                       .
      max_width := 5; // waere: 4-stellige Zahl + ' ' ..                                       .
      for i := A_to downto 1 do begin
         if (A[i] > 0) then begin
            max_width := Num_Digits(1.0 * i);
            Break;
         end;
      end;
   // --                                                                                       .
      if not(be_quiet) then begin
         if (A_cnt > 1) then begin
            WriteLn(A_cnt,' primes were found!');
         end else begin
            WriteLn(A_cnt,' prime was found!');
         end;
         WriteLn;
      end;
   // --                                                                                       .
      p_cnt := 1;
      for i := 1 to A_to do begin
         if (A[i] > 0) then begin
            if (A_cnt > 1) then begin
               Write(Format('%*.*d',[max_width,0,i]),', ');
            end else begin
               Write(Format('%*.*d',[max_width,0,i]),'.');
            end;
            if ((A_cnt > 1) and
                ((p_cnt mod (SCREEN_WIDTH div (2 + max_width))) = 0)) then begin
               WriteLn;
            end;
            Dec(A_cnt);
            Inc(p_cnt);
         end;
      end;
      WriteLn;
   // --                                                                                       .
   end else begin
      // Ungueltigen Parameter / Schalter gefunden? -> Ausgabe desjenigen welchen ..           .
      if (CmdLine_ok <> CMDLINE_VALID) then begin
         WriteLn('Sorry! Unknown switch: ','''',ParamStr(CmdLine_ok),'''','.');
         WriteLn;
      end;
      // Angegebene Kommandozeile war wohl niX .. Usage-Meldung ausgeben ..                    .
      WriteLn('Usage: ',Program_name,' [N] {-q|--quiet} | {-v|--version} | {-h|' +
         '--history}');
      WriteLn;
      WriteLn(' e.g.: ',Program_name,' 1000');
      WriteLn;
      WriteLn('  N                  A number between 2 (min) and ',
         fmtFormatNumber((1.0 * MAX_N),0),' (max).');
      WriteLn('  -q, /q, --quiet    Quiet: no additional output.');
      WriteLn('  -v, /v, --version  Display version number only.');
      WriteLn('  -h, /h, --history  Display program/ version history.');
      WriteLn;
      WriteLn('Hint(s): Parameters enclosed in ''[]'' are mandatory; parameters' +
         ' enclosed in ''{}'' are optional; parameters seperated by ''|'' exclu' +
         'de each other; ''<..>'' within a parameter means that *you* have to s' +
         'upply additional information.');
      WriteLn;
      WriteLn('Bug report: Pls. report all bug(s) and/ or wishes, suggestions e' +
         'tc. to <BugReport@nefzger.at>, including the module name (which is: ''',
         Program_name,''') and module version (which is: ''',resGetXVersionInfo(),
         ''' [''',resGetXWersionInfo(),''']','). Thank you.');
   end;
   Halt(0); // Normales Programmende, mit Exitcode 0 retour ..                                 .
except
   on e: Exception do begin
      {$INCLUDE '.\all.fatalerror.inc'}
      WriteLn(Error,' Fatal: Exception: ','''',e.Message,'''','; pls. contact t' +
         'he programmer: <BugReport@nefzger.at>.');
      Halt(1); // Abnormales Programmende: irgendwo wurde (intern) eine Exception geworfen ..  .
   end;
end; {try} end. // {begin}                                                                     .

